import sys
import traceback


def get_traceback(exception) -> str:
    """Get formatted traceback from exception."""

    # - Try to get traceback from better_exceptions

    try:
        import better_exceptions

        return "".join(better_exceptions.format_exception(*exception))
    except:
        pass

    # - Fallback to traceback

    return traceback.format_exc()


def test():
    try:
        raise Exception("test")
    except Exception:
        print(get_traceback(exception=sys.exc_info()))


if __name__ == "__main__":
    test()



import pytest

from loguru import logger
from retry import retry


class RetryLogger:
    """Retry logger for retry package. By default retry package logs to stderr. This class enables to log to loguru."""

    @staticmethod
    def warning(fmt, error, delay):
        logger.warning("Retrying", error=error, delay=delay)


@pytest.mark.slow
def test():
    try:
        retry(tries=3, delay=0.1, logger=RetryLogger)(lambda: 1 / 0)()
    except Exception as e:
        logger.exception("Failed", error=e)


if __name__ == "__main__":
    test()



import sys

from collections import OrderedDict
from datetime import datetime
from typing import Any

from loguru import logger

from pretty_json_loguru.get_traceback import get_traceback

try:
    from loguru import Record
except ImportError:
    # for some reason, Record does not import this way in loguru 0.6.0
    Record = Any

try:
    import ujson as json
except ImportError:
    import json


def format_as_colored_json(
    attach_raw_traceback: bool = True,
):
    """Loguru formatter builder for colored json logs.

    Sample output (colored in the console):
    {"ts": "2024-07-29 08:19:03.675", "module": "format_as_colored_json", "message": "Simple message"}
    {"ts": "2024-07-29 08:19:03.675", "module": "format_as_colored_json", "message": "Message with extra", "foo": "bar"}
    {"ts": "2024-07-29 08:19:03.675", "module": "format_as_colored_json", "message": "Exception caught", "error": "ValueError: This is an exception", "traceback": "...\nValueError: This is an exception"}

    Parameters
    ----------
    append_non_json_traceback : bool
        If True, extra traceback will be appended to the log, as if we use the vanilla formatter.

    """

    def _format_as_json_colored(record: Record):
        # - Validate record

        assert "_json" not in record["extra"]

        # - Pop extra

        extra = dict(record["extra"])
        extra.pop("source", None)

        # - Create record_dic that will be serialized as json

        record_dic = {
            "msg": record["message"],
            # "module": record["module"],
            "ts": datetime.fromisoformat(str(record["time"])).strftime(
                "%Y-%m-%d %H:%M:%S.%f"
            )[:-3],  # 2023-03-26 13:04:09.512
            "source": record["extra"].get("source", ""),
        }

        if record["exception"]:
            record_dic["traceback"] = get_traceback(
                exception=record["exception"]
            ).strip()
            record_dic["error"] = record_dic["traceback"].split("\n")[-1]

        record_dic = {k: v for k, v in record_dic.items() if v}
        record_dic.update(extra)

        # - Sort keys

        record_dic = OrderedDict(
            sorted(
                record_dic.items(),
                key=lambda kv: {
                    "ts": 0,
                    # "module": 1,
                    "msg": 2,
                    "source": 3,
                    "extra": 4,
                    "error": 5,
                    "traceback": 6,
                    "level": 7,
                }.get(kv[0], 4),  # default is extra
            )
        )

        # - Get json

        output = (
            json.dumps(
                record_dic,
                default=str,
                ensure_ascii=False,
            )
            .replace("{", "{{")
            .replace(
                "}",
                "}}",
            )
        )

        # - Iterate over json and add color tags

        for i, (key, value) in enumerate(record_dic.items()):
            # - Pick color

            color_key = {
                "ts": "green",
                "module": "cyan",
                "msg": "level",
                "error": "red",
                "traceback": "red",
            }.get(key, "magenta")

            color_value = {
                "ts": "green",
                "module": "cyan",
                "msg": "level",
                "error": "red",
                "traceback": "red",
            }.get(key, "yellow")

            # - Dump to json

            value_str = (
                json.dumps(value, default=str, ensure_ascii=False)
                .replace("{", "{{")
                .replace("}", "}}")
            )

            # - Add colors for keys and values

            output = output.replace(
                f'"{key}": {value_str}',
                f'<{color_key}>"{{extra[_extra_{2 * i}]}}"</{color_key}>: <{color_value}>{{extra[_extra_{2 * i + 1}]}}</{color_value}>',
            )

            # - Add key and value to record, from where loguru will get them

            if record:
                record["extra"][f"_extra_{2 * i}"] = key
                record["extra"][f"_extra_{2 * i + 1}"] = json.dumps(
                    value,
                    ensure_ascii=False,
                    default=str,
                )

        # - Add traceback on new line

        if attach_raw_traceback and "traceback" in record_dic:
            record["extra"]["_extra_traceback"] = record_dic["traceback"]
            output += "\n<red>{extra[_extra_traceback]}</red>"

        # - Add white color for the whole output

        return "<white>" + output + "\n" + "</white>"

    return _format_as_json_colored


def test():
    logger.info("Simple message")
    logger.info("Message with extra", foo="bar")

    try:
        raise ValueError("This is an exception")
    except ValueError:
        logger.exception("Exception caught")


if __name__ == "__main__":
    logger.remove()
    logger.add(sys.stdout, format=format_as_colored_json(attach_raw_traceback=False))
    test()



import json
import math
import sys

from loguru import logger

from pretty_json_loguru.get_traceback import get_traceback


def format_as_json(
    record: dict,
    attach_raw_traceback: bool = True,
) -> str:
    """Loguru json formatter.

    Sample output:
    {"level": "INFO", "message": "Simple message", "ts": 1722241041459, "source": "", "extra": {}, "stack": "", "error": ""}
    {"level": "INFO", "message": "Message with extra", "ts": 1722241041459, "source": "", "extra": {"extra": {"foo": "bar"}}, "stack": "", "error": ""}
    {"level": "ERROR", "message": "Exception caught", "ts": 1722241041459, "source": "", "extra": {}, "stack": "...\nValueError: This is an exception\n", "error": ""}

    """
    assert "_json" not in record["extra"]

    extra = dict(record["extra"])
    extra.pop("source", None)

    record_dic = {
        "level": record["level"].name,
        "message": record["message"],
        "ts": int(math.floor(record["time"].timestamp() * 1000)),  # epoch millis
        "source": record["extra"].get("source", ""),
        "extra": extra,
        "stack": "",
        "error": "",
    }

    if record["exception"]:
        record_dic["traceback"] = get_traceback(exception=record["exception"]).strip()
        record_dic["error"] = record_dic["stack"].split("\n")[-1]

    record["extra"]["_json"] = json.dumps(record_dic, default=str)

    output = "{extra[_json]}\n"

    if attach_raw_traceback and "traceback" in record_dic:
        record["extra"]["_extra_traceback"] = record_dic["traceback"]
        output += "\n<red>{extra[_extra_traceback]}</red>"

    return output


def test():
    logger.info("Simple message")
    logger.info("Message with extra", extra={"foo": "bar"})

    try:
        raise ValueError("This is an exception")
    except ValueError:
        logger.exception("Exception caught")


def example():
    logger.remove()
    logger.add(sys.stderr, format=format_as_json)
    test()

    logger.remove()
    logger.add(
        sys.stderr,
        format=lambda record: format_as_json(record, attach_raw_traceback=True),
    )
    test()


if __name__ == "__main__":
    example()



import os

import dony


@dony.command()
def release(
    version: str = None,
    uv_publish_token: str = None,
):
    """Bump version and publish to PyPI"""

    # - Select default arguments

    version = version or dony.select(
        "Choose version",
        choices=[
            "patch",
            "minor",
            "major",
        ],
    )

    uv_publish_token = uv_publish_token or dony.input(
        "Enter UV publish token (usually a PyPI token)",
        default=os.getenv("UV_PUBLISH_TOKEN", ""),
    )

    # - Get current branch

    original_branch = dony.shell(
        "git branch --show-current",
        quiet=True,
    )

    # - Go to main

    dony.shell("""

                # - Exit if there are staged changes

                git diff --cached --name-only | grep -q . && git stash

                # - Go to main

                git checkout main

                # - Git pull

                git pull
    """)

    # - Bump

    dony.shell(
        f"""

            # - Bump

            VERSION=$(uv version --bump {version} --short)
            echo $VERSION

            # - Commit, tag and push

            git add pyproject.toml
            git commit --message "chore: release-$VERSION"
            git tag --annotate "release-$VERSION" --message "chore: release-$VERSION" HEAD
            git push
            git push origin "release-$VERSION" # push tag to origin,
            """
    )

    # - Build and publish

    dony.shell(
        f"""
        uv build
        UV_PUBLISH_TOKEN={uv_publish_token} uv publish
        """
    )

    # - Go back to original branch

    dony.shell(
        f"""
        git checkout {original_branch}
        """
    )


if __name__ == "__main__":
    release()



import re

import dony


__NAME__ = "squash:0.1.4"


@dony.command()
def squash(
    new_branch: str = None,
    target_branch: str = None,
    commit_message: str = None,
    checkout_to_new_branch: str = None,
    remove_merged_branch: str = None,
):
    """Squashes current branch to main, checkouts to a new branch"""

    # - Get default branch if not set

    new_branch = (
        new_branch or f"workflow_{dony.shell('date +%Y%m%d_%H%M%S', quiet=True)}"
    )

    # - Get current branch

    merged_branch = dony.shell(
        "git branch --show-current",
        quiet=True,
    )

    # - Look for main/master branch

    main_or_master_branch = dony.shell(
        "git branch --list main | grep -q main && echo main || echo master",
        quiet=True,
    )

    # - Get target branch

    target_branch = target_branch or dony.input(
        "Enter target branch:",
        default=main_or_master_branch,
    )

    # - Do git diff

    dony.shell(
        f"""
        root=$(git rev-parse --show-toplevel)
        
        git diff {target_branch} --name-only -z \
        | while IFS= read -r -d '' file; do
            full="$root/$file"
            printf '\033[1;35m%s\033[0m\n' "$full"
            git --no-pager diff --color=always {target_branch} -- "$file" \
              | sed $'s/^/\t/'
            printf '\n'
          done
"""
    )

    # Ask user to confirm

    dony.confirm("Start squashing?", default=False)

    # - Check if target branch exists

    if (
        dony.shell(
            f"""
        git branch --list {target_branch}
    """
        )
        == ""
    ):
        return dony.error(f"Target branch {target_branch} does not exist")

    # - Get commit message from the user

    if not commit_message:
        while True:
            commit_message = dony.input(
                f"Enter commit message for merging branch {merged_branch} to {target_branch}:"
            )
            if bool(
                re.match(
                    r"^(?:(?:feat|fix|docs|style|refactor|perf|test|chore|build|ci|revert)(?:\([A-Za-z0-9_-]+\))?(!)?:)\s.+$",
                    commit_message.splitlines()[0],
                )
            ):
                break
            dony.print("Only conventional commits are allowed, try again")

    # - Check if user wants to checkout to a new branch

    checkout_to_new_branch = dony.confirm(
        f"Checkout to new branch {new_branch}?",
        provided_answer=checkout_to_new_branch,
    )

    # - Check if user wants to remove merged branch

    remove_merged_branch = dony.confirm(
        f"Remove merged branch {merged_branch}?",
        provided_answer=remove_merged_branch,
    )

    # - Do the process

    dony.shell(
        f"""

        # - Make up to date

        git diff --name-only | grep -q . && git stash push -m "squash-{merged_branch}"
        git checkout {target_branch}

        # - Set upstream if needed

        if ! git ls-remote --heads --exit-code origin "{target_branch}" >/dev/null; then
            git push --set-upstream origin {target_branch} --force
        fi

        # - Pull target branch

        git pull

        # - Merge

        git merge --squash {merged_branch}
        git commit -m "{commit_message}"
        git push 

        # - Remove merged branch

        if {str(remove_merged_branch).lower()}; then
            git branch -D {merged_branch}
            git push origin --delete {merged_branch}
        fi

        # - Create new branch

        if {str(checkout_to_new_branch).lower()}; then
            git checkout -b {new_branch}
            git push --set-upstream origin {new_branch}
        fi
    """,
    )


if __name__ == "__main__":
    squash()



import sys

from loguru import logger

from pretty_json_loguru.formatters.format_as_colored_json import (
    format_as_colored_json,
)


def setup_json_loguru(
    level: str = "DEBUG",
    attach_raw_traceback: bool = True,
    remove_existing_sinks: bool = True,
):
    """Set up loguru logger with JSON format (colored).

    Parameters
    ----------
    level : str
        Logging level
    attach_raw_traceback : bool
        If True, extra traceback will be appended to the log, as if we use the vanilla formatter.
    remove_existing_sinks : bool
        Whether to remove existing sinks

    """
    if remove_existing_sinks:
        logger.remove()

    logger.add(
        sink=sys.stdout,
        level=level,
        format=format_as_colored_json(
            attach_raw_traceback=attach_raw_traceback,
        ),
    )
